
	name	ABSDIO
	page	60,130

;***************************************************************************/
;*  Absloute disk read and write function. Non-standard IO			     */
;***************************************************************************/


absdio  segment para public 'PROG' 	;* For Lattice	*/
	assume	cs:absdio,ds:nothing,es:nothing


;***************************************************************************/
;   MSC calling sequence:	(also Lattice)
; err = absdio (oper, drive, buf, sector, count)
; int err;		/* Suucess/ Fail code */
; int oper;		/* Operation: 0 = read, One = write */
; int drive;		/* Drive to read from, 0 = 'A', 1 = 'B', etc */
; char *buf;		/* Transfer buffer location */
; int sector;		/* Starting Logical Sector address */
; int count;		/* Total sector count to read */
;
;   On exit: err (AX reg) contains zero if function executed ok.
;	     If error, AL will contain error code per the DOS Tech Ref
;            and AH will contain 80H to ensure a non-zero return value.
;            err should be checked as follows:
;
;	if (err) { err &= 0xFF; ....process error }
;
;
;***************************************************************************/
										
	public	_absdio 	; make entry point visible

_absdio proc	FAR		; assumes LARGE code model

	push	bp		; setup / save critical regs
        mov     bp, sp
        push    ds
        push    si
        push    di


	mov	al,byte ptr[bp+8]    ;get drive number
	lds	bx,dword ptr[bp+10]  ;get buffer loc (LARGE data model assumed)
	mov	dx,[bp+14]	     ;get starting sector num
	mov	cx,[bp+16]	     ;get sector count
	push	bp		     ;save bp just to be safe
	cmp	word ptr [bp+6],1    ;check read / write
	je	abswrt		     ;write request
	int	25h		     ;call DOS absolute read
	jmp	short wrapup	     ;done, go to wrapup
abswrt:					;write request
	int	26h			;call DOS absolute write
wrapup:					; wrapup function
	pop	si		;remove flags from stack
	pop     bp              ;recover bp
	mov	bl,al		;save error code
	jc	err_end 	;error return
	xor	ax,ax		;return value = 0
abs_ret:			;restore environment
        pop     di
        pop     si
        pop     ds
        pop     bp
        ret
err_end:			; set error reurn code
	mov	al,ah		; al = error code from DOS
	mov	ah,80h		; ensure code negative
	jmp	abs_ret 	; exit now

_absdio endp			; end procedure
absdio  ends			; end segment
	end			; end program
										
